@@ -30,6 +30,8 @@
 #endif
 #define UTMP_HACK
 #include <unistd.h>
+#include <sys/types.h>
+#include <sys/wait.h>
 
 static int cpid;
 static int size;
@@ -49,9 +51,8 @@ int kbuf[100];
 int kbuflen = 0;
 static unsigned char okbuf[100];
 static int okbuflen = 0;
-int sighit = 0;
 static int oldcr = 0, oldcc = 0, oldch = 0;
-char voices[7][64];
+char voices[TTS_SYNTH_COUNT][64];
 static int shell = 0;
 int special = 0;
 int cl_synth = 0;
@@ -115,7 +116,7 @@ static void child()
   }
   else
   {
-    (void) execl(usershell, arg, 0);
+    (void) execl(usershell, arg, (void *) 0);
   }
   perror("execl");
   exit(1);
@@ -221,9 +222,35 @@ static void utmpconv(char *s, char *d, int pid)
   {
     (void) kill(tts.pid, 9);
   }
+  if (cpid)
+  {
+    (void) kill(cpid, 9);
+  }
   exit(0);
 }
 
+static void getoutput();
+
+/*ARGSUSED*/ static void child_finish(int sig)
+{
+  int pid = 0;
+
+while (readable(master, 0)) getoutput();
+  (void) signal(SIGCHLD, &child_finish);
+  pid = waitpid(-1, NULL, WNOHANG);
+  /* It is possible for the child to die and for this handler to be called
+     before fork() ever returned, in which case cpid will still hold a value
+     of 0, so need to check for that. */
+  if ((pid == tts.pid) && tts.reinit)
+  {
+    tts_reinit2();
+  }
+  else if (cpid == 0 || pid == cpid)
+  {
+    finish(0);
+  }
+}
+
 static int is_char(int ch)
 {
 #ifdef __linux__
@@ -236,12 +263,33 @@ static int is_char(int ch)
 #endif
 }
 
+static int is_separator(int ch)
+{
+  int result=0;
+  if ((ch != ch % 0xFF) || isspace(ch))
+  {
+    result=1;
+  }
+  else if (((ch >= 0x21) && (ch <= 0x2F))
+	   || ((ch >= 0x3A) && (ch <= 0x40))
+	   || ((ch >= 0x5B) && (ch <= 0x60))
+	   || ((ch >= 0x7B) && (ch <= 0x7E))
+	   )
+  {
+    result=1;
+  }
+  return result;
+}
+
+
+
+
 static void getinput()
 {
   int key;
 
   size = read(0, buf, 255);
-  if (!size)
+  if (size <= 0)
   {
     finish(0);
   }
@@ -301,7 +349,7 @@ static void getinput()
     ui.meta = 0;
     return;
   }
-  if (ui.kbsay == 2 && is_char(key))
+  if (ui.kbsay == 2 && is_separator(key))
   {
     tts_out(okbuf, okbuflen);
     okbuflen = tts.oflag = 0;
@@ -397,7 +445,7 @@ static void win_rlf()
   if (win->cr == win_scrollmin) win_scrolldown(); else win->cr--;
 }
 
-static int readable(int fd, int wait)
+int readable(int fd, int wait)
 {
   fd_set fds;
   struct timeval tv;
@@ -443,6 +491,12 @@ static void kbsay()
   if (!ui.kbsay) return;
   if (buf[0] == 8 || kbuf[0] == 127)
   {
+    if ((ui.kbsay == 2) 
+	&& (okbuflen!=0))
+    {
+      okbuf[--okbuflen] = 0;
+    }
+
     /*tts_say(_("back")); */
     return;
   }
@@ -1015,12 +1069,12 @@ static void getoutput()
 	}
       }
 #endif
-      if (ch == kbuf[0] && win->cr == oldcr && win->cc == oldcc && kbuflen)
+      if (ch == (char)kbuf[0] && win->cr == oldcr && win->cc == oldcc && kbuflen)
       {
 /* this character was (probably) echoed as a result of a keystroke */
 	kbsay();
 	win_addchr(ch, 0);
-	(void) memmove(kbuf + sizeof(int), kbuf, (--kbuflen) * sizeof(int));
+	(void) memmove(kbuf, kbuf + 1, (--kbuflen) * sizeof(int));
       }
       else
       {
@@ -1083,7 +1137,9 @@ static void getoutput()
       {
 	break;
       }
-      if (kbuf[0] == 0x1b5b43 || ui.curtrack == 2)
+      if (kbuf[0] == 0x1b5b43 || 
+	  ((ui.curtrack == 2) 
+	   && (ui.kbsay != 2 || is_separator(kbuf[0]))))
       {
 	ui_saychar(win->cr, win->cc);
       }
@@ -1173,6 +1229,7 @@ static void getoutput()
 
 static void get_tts_input()
 {
+  if (!readable(tts.fd, 1)) return;
   (void) read(tts.fd, buf, 100);
 }
 
@@ -1182,31 +1239,32 @@ static void parent()
   struct termios rt;
   int maxfd;
 
-  (void) tcgetattr(0, &t);
   (void) memcpy(&rt, &t, sizeof(struct termios));
   cfmakeraw(&rt);
   rt.c_cc[VMIN] = 1;
   rt.c_cc[VTIME] = 0;
   (void) tcsetattr(0, TCSAFLUSH, &rt);
-  (void) signal(SIGCHLD, &finish);
   yasr_ttyname_r(0, (char *) (buf + 128), 32);
   yasr_ttyname_r(slave, (char *) (buf + 192), 32);
   utmpconv((char *) (buf + 128), (char *) (buf + 192), cpid);
   maxfd = (master > tts.fd ? master : tts.fd) + 1;
 
-  FD_ZERO(&rf);
   for (;;)
   {
+    int result;
+    FD_ZERO(&rf);
     FD_SET(master, &rf);
     FD_SET(0, &rf);
     FD_SET(tts.fd, &rf);
-    (void) select(maxfd, &rf, NULL, NULL, NULL);
-
-/* tbc - can we replace use of sighit by checking the select return val? */
-    if (sighit)
+    result = select(maxfd, &rf, NULL, NULL, NULL);
+    if (result == -1)
     {
-      sighit = 0;
-      continue;
+      if (errno == EINTR) continue;
+      else
+      {
+	perror("select");
+	break;
+      }
     }
     if (FD_ISSET(0, &rf))
     {
@@ -1263,7 +1321,7 @@ int main(int argc, char *argv[])
   uinit();
   (void) memset(&tts, 0, sizeof(Tts));
   opt_init();
-  (void) memset(voices, 0, 128);
+  (void) memset(voices, 0, sizeof(voices));
   while (flag)
   {
     switch (getopt(argc, argv, "C:c:s:p:"))
@@ -1311,6 +1369,8 @@ int main(int argc, char *argv[])
 #endif
 
   (void) openpty(&master, &slave, NULL, NULL, &winsz);
+  (void) signal(SIGCHLD, &child_finish);
+  (void) tcgetattr(0, &t);
   cpid = fork();
   if (cpid > 0) parent();
   else if (cpid == 0) child();
@@ -1326,7 +1386,8 @@ static Win *wininit(int nr, int nc)
   win = (Win *) calloc(sizeof(Win), 1);
   if (!win)
   {
-    fprintf(stderr, "wininit: cannot allocate %ld bytes\n", sizeof(Win));
+    fprintf(stderr, "wininit: cannot allocate %lu bytes\n",
+	    (unsigned long) sizeof(Win));
     exit(1);
   }
   win->rows = nr;
@@ -1379,7 +1440,7 @@ void speak(char *ibuf, int len)
 	nc++;
       }
       tts_saychar(lc);
-      tts_out((unsigned char *) obuf, sprintf(obuf, "repeats %d times\r", nc + 1));
+      tts_out((unsigned char *) obuf, sprintf(obuf, "%s %d %s\r",_("repeats"), nc + 1,_("times")));
       olen = nc = 0;
     }
 	else
