@@ -111,6 +111,7 @@ void tts_flush()
  
 }
 
+#if 0
 static void tts_wait(int usecs)
 {
   char buf[100];
@@ -121,6 +122,7 @@ static void tts_wait(int usecs)
     read(tts.fd, buf, sizeof(buf));
   }
 }
+#endif
 
 void tts_silence()
 {
@@ -624,7 +626,6 @@ int tts_init( int first_call)
     char *logname = getenv("LOGNAME");
     if (logname == NULL) logname = getlogin();
     tts_printf_ll("SET self CLIENT_NAME %s:yasr:tts\r\n", logname);
-    tts_send(buf, strlen(buf));
   }
   else tts_send(synth[tts.synth].init, strlen(synth[tts.synth].init));
 
