@@ -25,6 +25,10 @@
 #define TTS_DOUBLETALK       3
 #define TTS_BNS              4
 #define TTS_APOLLO           5
+#define TTS_FESTIVAL	     6	
+#define TTS_Ciber232         7
+#define TTS_SPEECHD          8
+#define TTS_SYNTH_COUNT      9
 
 typedef struct Tts_synth Tts_synth;
 
